@@ -141,13 +141,12 @@ group_by_prepare <- function(.data, ..., .add = FALSE, .dots = deprecated(), add
     .add <- add
   }
 
-  new_groups <- enquos(...)
+  new_groups <- enquos(..., .ignore_empty = "all")
   if (!missing(.dots)) {
     # Used by dbplyr 1.4.2 so can't aggressively deprecate
     lifecycle::deprecate_warn("1.0.0", "group_by(.dots = )")
     new_groups <- c(new_groups, compat_lazy_dots(.dots, env = caller_env()))
   }
-  new_groups <- new_groups[!map_lgl(new_groups, quo_is_missing)]
 
   # If any calls, use mutate to add new columns, then group by those
   computed_columns <- add_computed_columns(ungroup(.data), new_groups, "group_by")
