@@ -22,29 +22,34 @@ tbl <- function(src, ...) {
 #' @param object to test/coerce.
 #' @param ... For `tbl()`, other fields used by class. For `as.tbl()`,
 #'   other arguments passed to methods.
-#' @examples
-#' as.tbl(mtcars)
 make_tbl <- function(subclass, ...) {
   subclass <- paste0("tbl_", subclass)
   structure(list(...), class = c(subclass, "tbl"))
 }
 
 #' @rdname tbl
+#' @param x Any object
 #' @export
 is.tbl <- function(x) inherits(x, "tbl")
 
-#' @export
-#' @rdname tbl
-#' @param x an object to coerce to a `tbl`
-as.tbl <- function(x, ...) UseMethod("as.tbl")
+tbl_vars_dispatch <- function(x) {
+  UseMethod("tbl_vars")
+}
 
-#' @export
-as.tbl.tbl <- function(x, ...) x
+new_sel_vars <- function(vars, group_vars) {
+  structure(
+    vars,
+    groups = group_vars,
+    class = c("dplyr_sel_vars", "character")
+  )
+}
 
 #' List variables provided by a tbl.
 #'
 #' `tbl_vars()` returns all variables while `tbl_nongroup_vars()`
-#' returns only non-grouping variables.
+#' returns only non-grouping variables. The `groups` attribute
+#' of the object returned by `tbl_vars()` is a character vector of the
+#' grouping columns.
 #'
 #' @export
 #' @param x A tbl object
@@ -52,25 +57,23 @@ as.tbl.tbl <- function(x, ...) x
 #'   variables.
 #' @keywords internal
 tbl_vars <- function(x) {
+  return(new_sel_vars(tbl_vars_dispatch(x), group_vars(x)))
+
+  # For roxygen and static analysis
   UseMethod("tbl_vars")
 }
+
+#' @export
+tbl_vars.data.frame <- function(x) {
+  names(x)
+}
+
 #' @rdname tbl_vars
 #' @export
 tbl_nongroup_vars <- function(x) {
   setdiff(tbl_vars(x), group_vars(x))
 }
 
-# Should `tbl_vars()` return this object?
-sel_vars <- function(x) {
-  vars <- tbl_vars(x)
-  group_vars <- group_vars(x)
-
-  structure(
-    vars,
-    class = "dplyr_sel_vars",
-    groups = group_vars
-  )
-}
 is_sel_vars <- function(x) {
   inherits(x, "dplyr_sel_vars")
 }
